@@ -12,6 +12,7 @@ NUMA node masks from :linux:`include/linux/nodemask.h`.
 from typing import Iterator
 
 from drgn import IntegerLike, Object, Program
+from drgn.helpers.common.prog import takes_program_or_default
 from drgn.helpers.linux.bitops import for_each_set_bit, test_bit
 
 __all__ = (
@@ -36,24 +37,26 @@ def for_each_node_mask(mask: Object) -> Iterator[int]:
     return for_each_set_bit(mask.bits, nr_node_ids)
 
 
+@takes_program_or_default
 def for_each_node_state(prog: Program, state: IntegerLike) -> Iterator[int]:
     """
     Iterate over all NUMA nodes in the given state.
 
     :param state: ``enum node_states`` (e.g., ``N_NORMAL_MEMORY``)
     """
-    mask = prog["node_states"][state]
-    return for_each_node_mask(mask)
+    return for_each_node_mask(prog["node_states"][state])
 
 
+@takes_program_or_default
 def for_each_node(prog: Program) -> Iterator[int]:
     """Iterate over all possible NUMA nodes."""
-    return for_each_node_state(prog, prog["N_POSSIBLE"])
+    return for_each_node_state(prog["N_POSSIBLE"])
 
 
+@takes_program_or_default
 def for_each_online_node(prog: Program) -> Iterator[int]:
     """Iterate over all online NUMA nodes."""
-    return for_each_node_state(prog, prog["N_ONLINE"])
+    return for_each_node_state(prog["N_ONLINE"])
 
 
 def node_state(node: IntegerLike, state: Object) -> bool:
