@@ -25,8 +25,7 @@ enum {
 	C_TOKEN_BOOL,
 	C_TOKEN_FLOAT,
 	C_TOKEN_DOUBLE,
-	C_TOKEN_COMPLEX,
-	MAX_SPECIFIER_TOKEN = C_TOKEN_COMPLEX,
+	MAX_SPECIFIER_TOKEN = C_TOKEN_DOUBLE,
 	MIN_QUALIFIER_TOKEN,
 	C_TOKEN_CONST = MIN_QUALIFIER_TOKEN,
 	C_TOKEN_RESTRICT,
