@@ -1,7 +1,7 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
-#include "drgn.h"
+#include "drgn_internal.h"
 #include "lexer.h"
 
 DEFINE_VECTOR_FUNCTIONS(drgn_token_vector);
