@@ -6,7 +6,8 @@
 
 #include "drgn_internal.h"
 
-struct drgn_debug_info_load_state;
+struct drgn_debug_info_options;
+struct drgn_standard_debug_info_find_state;
 
 struct drgn_error *drgn_program_finish_set_kernel(struct drgn_program *prog);
 
@@ -24,7 +25,17 @@ struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
 struct drgn_error *read_vmcoreinfo_fallback(struct drgn_program *prog);
 
 struct drgn_error *
-linux_kernel_report_debug_info(struct drgn_debug_info_load_state *load);
+linux_kernel_loaded_module_iterator_create(struct drgn_program *prog,
+					   struct drgn_module_iterator **ret);
+
+struct drgn_error *
+drgn_module_try_vmlinux_files(struct drgn_module *module,
+			      const struct drgn_debug_info_options *options);
+
+struct drgn_error *
+drgn_module_try_linux_kmod_files(struct drgn_module *module,
+				 const struct drgn_debug_info_options *options,
+				 struct drgn_standard_debug_info_find_state *state);
 
 #define KDUMP_SIGNATURE "KDUMP   "
 #define KDUMP_SIG_LEN (sizeof(KDUMP_SIGNATURE) - 1)
