@@ -559,6 +559,7 @@ static DrgnObject *DrgnObject_new(PyTypeObject *subtype, PyObject *args,
 
 static void DrgnObject_dealloc(DrgnObject *self)
 {
+	PyObject_GC_UnTrack(self);
 	Py_DECREF(DrgnObject_prog(self));
 	drgn_object_deinit(&self->obj);
 	Py_TYPE(self)->tp_free((PyObject *)self);
@@ -1635,6 +1636,12 @@ static PyMappingMethods DrgnObject_as_mapping = {
 	.mp_subscript = (binaryfunc)DrgnObject_subscript,
 };
 
+static int DrgnObject_traverse(DrgnObject *self, visitproc visit, void *arg)
+{
+	Py_VISIT(DrgnObject_prog(self));
+	return 0;
+}
+
 PyTypeObject DrgnObject_type = {
 	PyVarObject_HEAD_INIT(NULL, 0)
 	.tp_name = "_drgn.Object",
@@ -1645,7 +1652,8 @@ PyTypeObject DrgnObject_type = {
 	.tp_as_mapping = &DrgnObject_as_mapping,
 	.tp_str = (reprfunc)DrgnObject_str,
 	.tp_getattro = (getattrofunc)DrgnObject_getattro,
-	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
+	.tp_traverse = (traverseproc)DrgnObject_traverse,
 	.tp_doc = drgn_Object_DOC,
 	.tp_richcompare = DrgnObject_richcompare,
 	.tp_iter = (getiterfunc)DrgnObject_iter,
@@ -1729,10 +1737,18 @@ DrgnObject *DrgnObject_container_of(PyObject *self, PyObject *args,
 
 static void ObjectIterator_dealloc(ObjectIterator *self)
 {
+	PyObject_GC_UnTrack(self);
 	Py_DECREF(self->obj);
 	Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
+static int ObjectIterator_traverse(ObjectIterator *self, visitproc visit,
+				   void *arg)
+{
+	Py_VISIT(self->obj);
+	return 0;
+}
+
 static DrgnObject *ObjectIterator_next(ObjectIterator *self)
 {
 	if (self->index >= self->length)
@@ -1756,7 +1772,8 @@ PyTypeObject ObjectIterator_type = {
 	.tp_name = "_drgn._ObjectIterator",
 	.tp_basicsize = sizeof(ObjectIterator),
 	.tp_dealloc = (destructor)ObjectIterator_dealloc,
-	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
+	.tp_traverse = (traverseproc)ObjectIterator_traverse,
 	.tp_iter = PyObject_SelfIter,
 	.tp_iternext = (iternextfunc)ObjectIterator_next,
 	.tp_methods = ObjectIterator_methods,
