@@ -18,6 +18,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define _unused_ __attribute__((__unused__))
+
 #ifndef LIBDRGN_PUBLIC
 #define LIBDRGN_PUBLIC __attribute__((__visibility__("default")))
 #endif
@@ -69,6 +71,10 @@
 	_Generic(sizeof(struct { _Static_assert(assert_expression, message); int _; }),\
 		 default: (eval_expression))
 
+#define sizeof_member(type, member) sizeof(((type *)0)->member)
+
+#define typeof_member(type, member) typeof(((type *)0)->member)
+
 #define container_of(ptr, type, member)				\
 static_assert_expression(					\
 	types_compatible(*(ptr), ((type *)0)->member)		\
@@ -127,6 +133,18 @@ static inline void *malloc64(uint64_t size)
 	return malloc(size);
 }
 
+// glibc added reallocarray() in 2.26, but since it's so trivial, it's easier to
+// duplicate it here than it is to do feature detection.
+static inline void *realloc_array(void *ptr, size_t nmemb, size_t size)
+{
+       size_t bytes;
+       if (__builtin_mul_overflow(nmemb, size, &bytes)) {
+               errno = ENOMEM;
+               return NULL;
+       }
+       return realloc(ptr, bytes);
+}
+
 static inline void *memdup(const void *ptr, size_t size)
 {
 	void *copy = malloc(size);
