@@ -233,7 +233,6 @@ def gen_tests_dwarf_py(dwarf_constants: Sequence[DwarfConstantType], f: TextIO)
 # Generated by scripts/gen_dwarf_constants.py.
 
 import enum
-from typing import Text
 """
     )
     for constant_type in dwarf_constants:
@@ -246,16 +245,6 @@ from typing import Text
             if name == "name":
                 f.write("  # type: ignore")
             f.write("\n")
-        f.write(
-            f"""
-    @classmethod
-    def str(cls, value: int) -> Text:
-        try:
-            return f"{constant_type.name}_{{cls(value).name}}"
-        except ValueError:
-            return hex(value)
-"""
-        )
 
 
 def main() -> None:
