@@ -1,11 +1,18 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
+import contextlib
 import os
 
 from drgn import NULL, cast
 from drgn.helpers.linux.fs import fget
-from drgn.helpers.linux.kernfs import kernfs_name, kernfs_path, kernfs_walk
+from drgn.helpers.linux.kernfs import (
+    kernfs_name,
+    kernfs_parent,
+    kernfs_path,
+    kernfs_root,
+    kernfs_walk,
+)
 from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import LinuxKernelTestCase
 
@@ -16,6 +23,29 @@ class TestKernfs(LinuxKernelTestCase):
         file = fget(find_task(cls.prog, os.getpid()), fd)
         return cast("struct kernfs_node *", file.f_inode.i_private)
 
+    def test_kernfs_parent(self):
+        with contextlib.ExitStack() as exit_stack:
+            fd = os.open("/sys/kernel/vmcoreinfo", os.O_RDONLY)
+            exit_stack.callback(os.close, fd)
+            dfd = os.open("/sys/kernel", os.O_RDONLY)
+            exit_stack.callback(os.close, dfd)
+            self.assertEqual(
+                kernfs_parent(self.kernfs_node_from_fd(fd)),
+                self.kernfs_node_from_fd(dfd),
+            )
+
+    def test_kernfs_root(self):
+        for path in ("/sys", "/sys/kernel", "/sys/kernel/vmcoreinfo"):
+            with self.subTest(path=path):
+                fd = os.open(path, os.O_RDONLY)
+                try:
+                    self.assertEqual(
+                        kernfs_root(self.kernfs_node_from_fd(fd)),
+                        self.prog["sysfs_root"],
+                    )
+                finally:
+                    os.close(fd)
+
     def test_kernfs_name(self):
         with open("/sys/kernel/vmcoreinfo", "r") as f:
             kn = self.kernfs_node_from_fd(f.fileno())
