@@ -38,6 +38,7 @@
 #include <linux/stackdepot.h>
 #endif
 #include <linux/sysfs.h>
+#include <linux/timekeeping.h>
 #include <linux/vmalloc.h>
 #include <linux/wait.h>
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
@@ -47,6 +48,66 @@
 #define HAVE_XARRAY 0
 #endif
 
+// Page pools were added in Linux kernel commit ff7d6b27f894 ("page_pool:
+// refurbish version of page_pool code") (in v4.18) and may not be enabled.
+#ifdef CONFIG_PAGE_POOL
+#define HAVE_PAGE_POOL 1
+// The header file was moved in Linux kernel commit a9ca9f9ceff3 ("page_pool:
+// split types and declarations from page_pool.h") (in v6.6).
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
+#include <net/page_pool/helpers.h>
+#else
+#include <net/page_pool.h>
+#endif
+#else
+#define HAVE_PAGE_POOL 0
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 18, 0)
+// These were added in b9ff604cff11 ("timekeeping: Add
+// ktime_get_coarse_with_offset") (in v4.18-rc1).
+static inline ktime_t ktime_get_coarse_boottime(void)
+{
+	struct timespec64 ts = get_monotonic_coarse64();
+
+	return ktime_mono_to_any(timespec64_to_ktime(ts), TK_OFFS_BOOT);
+}
+
+static inline ktime_t ktime_get_coarse_clocktai(void)
+{
+	struct timespec64 ts = get_monotonic_coarse64();
+
+	return ktime_mono_to_any(timespec64_to_ktime(ts), TK_OFFS_TAI);
+}
+
+// These were added in Linux kernel commit 06aa376903b6 ("timekeeping: Add more
+// coarse clocktai/boottime interfaces") (in v4.18).
+static inline time64_t ktime_get_boottime_seconds(void)
+{
+	return ktime_divns(ktime_get_coarse_boottime(), NSEC_PER_SEC);
+}
+
+static inline time64_t ktime_get_clocktai_seconds(void)
+{
+	return ktime_divns(ktime_get_coarse_clocktai(), NSEC_PER_SEC);
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 3, 0)
+// These were added in 4c54294d01e6 ("timekeeping: Add missing _ns functions for
+// coarse accessors") (in v5.3).
+static inline u64 ktime_get_coarse_boottime_ns(void)
+{
+	return ktime_to_ns(ktime_get_coarse_boottime());
+}
+
+static inline u64 ktime_get_coarse_clocktai_ns(void)
+{
+	return ktime_to_ns(ktime_get_coarse_clocktai());
+}
+#endif
+
+
 // Convert a 4-character string to a seed for drgn_test_prng32().
 static inline u32 drgn_test_prng32_seed(const char *s)
 {
@@ -475,6 +536,56 @@ static void drgn_test_net_exit(void)
 	dev_put(drgn_test_netdev);
 }
 
+// page_pool
+
+const int drgn_test_have_page_pool = HAVE_PAGE_POOL;
+
+#if HAVE_PAGE_POOL
+struct page_pool *drgn_test_page_pool;
+struct page *drgn_test_page_pool_page;
+#endif
+
+static int drgn_test_page_pool_init(void)
+{
+#if HAVE_PAGE_POOL
+	struct page_pool_params params = {
+		.order = 0,
+		.flags = 0,
+		.pool_size = 1,
+		.nid = NUMA_NO_NODE,
+	};
+	struct page_pool *pool;
+
+	pool = page_pool_create(&params);
+	if (IS_ERR(pool))
+		return PTR_ERR(pool);
+	drgn_test_page_pool = pool;
+
+	drgn_test_page_pool_page = page_pool_alloc_pages(pool, GFP_KERNEL);
+	if (!drgn_test_page_pool_page)
+		return -ENOMEM;
+#endif
+	return 0;
+}
+
+static void drgn_test_page_pool_exit(void)
+{
+#if HAVE_PAGE_POOL
+	if (drgn_test_page_pool_page) {
+		// page_pool_put_page() changed in Linux kernel commit
+		// 458de8a97f10 ("net: page_pool: API cleanup and comments") (in
+		// v5.7).
+		page_pool_put_page(drgn_test_page_pool,
+				   drgn_test_page_pool_page,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 7, 0)
+				   0,
+#endif
+				   true);
+	}
+	page_pool_destroy(drgn_test_page_pool);
+#endif
+}
+
 // percpu
 
 DEFINE_PER_CPU(u32, drgn_test_percpu_static);
@@ -1231,6 +1342,9 @@ int drgn_test_function(int x)
 	return x + 1;
 }
 
+char drgn_test_data[] = "abc";
+const char drgn_test_rodata[] = "def";
+
 // kmodify
 
 #ifdef __x86_64__
@@ -1430,9 +1544,55 @@ static ssize_t drgn_test_crash_store(struct kobject *kobj,
 static struct kobj_attribute drgn_test_crash_attr =
 	__ATTR(crash, 0200, NULL, drgn_test_crash_store);
 
+static ssize_t drgn_test_boottime_seconds_show(struct kobject *kobj,
+					       struct kobj_attribute *attr,
+					       char *buf)
+{
+	return sprintf(buf, "%lld\n", ktime_get_boottime_seconds());
+}
+
+static struct kobj_attribute drgn_test_boottime_seconds_attr =
+	__ATTR(boottime_seconds, 0444, drgn_test_boottime_seconds_show, NULL);
+
+static ssize_t drgn_test_coarse_boottime_ns_show(struct kobject *kobj,
+						 struct kobj_attribute *attr,
+						 char *buf)
+{
+	return sprintf(buf, "%llu\n", ktime_get_coarse_boottime_ns());
+}
+
+static struct kobj_attribute drgn_test_coarse_boottime_ns_attr =
+	__ATTR(coarse_boottime_ns, 0444, drgn_test_coarse_boottime_ns_show,
+	       NULL);
+
+static ssize_t drgn_test_clocktai_seconds_show(struct kobject *kobj,
+					       struct kobj_attribute *attr,
+					       char *buf)
+{
+	return sprintf(buf, "%lld\n", ktime_get_clocktai_seconds());
+}
+
+static struct kobj_attribute drgn_test_clocktai_seconds_attr =
+	__ATTR(clocktai_seconds, 0444, drgn_test_clocktai_seconds_show, NULL);
+
+static ssize_t drgn_test_coarse_clocktai_ns_show(struct kobject *kobj,
+						 struct kobj_attribute *attr,
+						 char *buf)
+{
+	return sprintf(buf, "%llu\n", ktime_get_coarse_clocktai_ns());
+}
+
+static struct kobj_attribute drgn_test_coarse_clocktai_ns_attr =
+	__ATTR(coarse_clocktai_ns, 0444, drgn_test_coarse_clocktai_ns_show,
+	       NULL);
+
 static struct attribute_group drgn_test_attr_group = {
 	.attrs = (struct attribute *[]){
 		&drgn_test_crash_attr.attr,
+		&drgn_test_boottime_seconds_attr.attr,
+		&drgn_test_coarse_boottime_ns_attr.attr,
+		&drgn_test_clocktai_seconds_attr.attr,
+		&drgn_test_coarse_clocktai_ns_attr.attr,
 		NULL,
 	},
 };
@@ -1465,6 +1625,7 @@ static void drgn_test_exit(void)
 	drgn_test_maple_tree_exit();
 	drgn_test_mm_exit();
 	drgn_test_net_exit();
+	drgn_test_page_pool_exit();
 	drgn_test_stack_trace_exit();
 	drgn_test_radix_tree_exit();
 	drgn_test_xarray_exit();
@@ -1486,6 +1647,9 @@ static int __init drgn_test_init(void)
 	if (ret)
 		goto out;
 	ret = drgn_test_net_init();
+	if (ret)
+		goto out;
+	ret = drgn_test_page_pool_init();
 	if (ret)
 		goto out;
 	ret = drgn_test_percpu_init();
