@@ -28,6 +28,7 @@ _ROOTFS_PACKAGES = [
     "libdw-dev",
     "libelf-dev",
     "libkdumpfile-dev",
+    "liblzma-dev",
     "libtool",
     "make",
     "pkgconf",
@@ -37,6 +38,7 @@ _ROOTFS_PACKAGES = [
     "python3-setuptools",
     # Test dependencies.
     "btrfs-progs",
+    "check",
     "iproute2",
     "kexec-tools",
     "kmod",
